"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const devkit_1 = require("@nrwl/devkit");
const OpenAPIParser = require("@readme/openapi-parser");
const camelcase_1 = require("camelcase");
const dashify = require("dashify");
const path = require("path");
const get_sdk_id_1 = require("../../helpers/get-sdk-id");
const generator_1 = require("../service-command-generator/generator");
const to_uppercase_1 = require("../util/to-uppercase");
const create_command_1 = require("../../helpers/create-command");
async function default_1(tree, schema) {
    var _a, _b, _c;
    console.log('generator running');
    const openApiSpec = (await OpenAPIParser.dereference(schema.spec));
    console.log('openApiSpec', openApiSpec.info);
    const spec = (0, get_sdk_id_1.parseOpenApiSpec)(openApiSpec);
    const sdkId = spec.info['x-sdk-json'].sdkId;
    const outputDirectory = path.join('.', `client-${sdkId}`);
    const packageName = `@field123/client-${sdkId}`;
    const commandSpecs = (0, create_command_1.createCommandSpecs)(openApiSpec);
    console.log('tree wdwadaawd: ', tree.children(`./${outputDirectory}/package.json`));
    let pkgVersion = '0.0.0';
    if (tree.exists(`./${outputDirectory}/package.json`)) {
        const currentPkgVersion = (_a = tree
            .read(`./${outputDirectory}/package.json`)) === null || _a === void 0 ? void 0 : _a.toString();
        console.log('packagejson: ', JSON.parse(currentPkgVersion));
        pkgVersion = (_c = (_b = JSON.parse(currentPkgVersion)) === null || _b === void 0 ? void 0 : _b.version) !== null && _c !== void 0 ? _c : '0.0.0';
    }
    const commandNames = getCommandNames(commandSpecs);
    (0, devkit_1.generateFiles)(tree, path.join(__dirname, 'files'), outputDirectory, {
        packageName,
        name: sdkId,
        pkgVersion,
        uppercase: to_uppercase_1.uppercase,
        camelCase: camelcase_1.default,
        dashify,
        serviceNamePascal: (0, camelcase_1.default)(sdkId, { pascalCase: true }),
        serviceNameCamel: (0, camelcase_1.default)(sdkId),
        serviceNameDash: dashify(sdkId),
        commands: commandNames
            .map((id) => ({
            dash: dashify(id),
            camel: (0, camelcase_1.default)(id),
            pascal: (0, camelcase_1.default)(id, { pascalCase: true })
        }))
            .sort((a, b) => sortCommandNamesAsc(a.dash, b.dash))
    });
    await (0, devkit_1.formatFiles)(tree);
    for await (const commandSpec of commandSpecs) {
        const { method, operationObj } = commandSpec;
        if (method === 'get') {
            await (0, generator_1.default)(tree, {
                name: operationObj.operationId,
                serviceName: sdkId,
                outputDirectory,
                getItem: commandSpec
            });
        }
    }
    return () => {
        // installPackagesTask(tree);
    };
}
exports.default = default_1;
function sortCommandNamesAsc(a, b) {
    return a.localeCompare(b);
}
function getCommandNames(commandSpecs) {
    return commandSpecs.reduce((acc, x) => {
        const id = x.operationObj.operationId;
        return [...acc, ...(id ? [id] : [])];
    }, []);
}
